Package: BicARE
Version: 1.69.0
Date: 2008-06-05
Title: Biclustering Analysis and Results Exploration
Depends: R (>= 1.8.0), Biobase (>= 2.5.5), multtest, GSEABase, GO.db
Imports: methods
Suggests: hgu95av2
Author: Pierre Gestraud
Maintainer: Pierre Gestraud <pierre.gestraud@curie.fr>
Description: Biclustering Analysis and Results Exploration.
License: GPL-2
URL: http://bioinfo.curie.fr
biocViews: Microarray, Transcription, Clustering
git_url: https://git.bioconductor.org/packages/BicARE
git_branch: devel
git_last_commit: 8dec4bf
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Packaged: 2025-10-31 20:44:11 UTC; biocbuild

useDynLib(BicARE)
export(FLOC, bicluster, testAnnot, testSet, residue, makeReport)
S3method(print, biclustering)
S3method(plot, bicluster)
importFrom("grDevices", "dev.off", "png")
importFrom("graphics", "axis", "matplot", "par")
importFrom("methods", "new")
importFrom("stats", "chisq.test", "phyper", "runif", "var")
importFrom("utils", "browseURL")
import("Biobase")
import("GSEABase")
import("multtest")
import("GO.db")

makeReport <- function(dirPath, dirName, resBic, browse=TRUE){
    
    if(substr(dirPath, start=nchar(dirPath), stop=nchar(dirPath)) == "/") dirPath <- substr(dirPath, start=1, stop=nchar(dirPath)-1)
    
    outDir <- file.path(dirPath, dirName)
    dir.create(outDir, showWarnings=FALSE)
    BicAREPath <- file.path(attr(as.environment(match("package:BicARE", search())), "path"), "report")
    file.copy(c(list.files(BicAREPath, full.names=T)), outDir, overwrite=T)
    
    nbBic <- nrow(resBic$mat.resvol.bic)
    
    fileHome <- file.path(outDir, "home.html")
    
    contentHome <- readLines(fileHome, warn=FALSE)
    contentHome <- sub("DATE1", resBic$param[5,2], contentHome)
    contentHome <- sub("DATE2", date(), contentHome)
    contentHome <- sub("ANALYSISNAME", dirName, contentHome)
    contentHome <- sub("NBBIC", resBic$param[1,2], contentHome)
    contentHome <- sub("RESIDU", resBic$param[2,2], contentHome)
    contentHome <- sub("PROBAINIT", resBic$param[3,2], contentHome)
    contentHome <- sub("NBITE", resBic$param[4,2], contentHome)
    
    ## array type, if present
    Data <- resBic$ExpressionSet
    modPuce <- FALSE
    if ((length(annotation(Data))!=0) && (require(paste(annotation(Data), ".db", sep=""), character.only=TRUE))) modPuce <- TRUE
    
    ## biclusters
    fileListBic <- file.path(outDir, "listBic.html")
    contentListBic <- readLines(fileListBic, warn=FALSE)
    tab <- ""
    for (i in 1:nbBic){
        tabListGenes <- "<hr><br>\n<table>\n \t<tr><th></th><th>Symbol</th></tr>\n"
        tabListSamples <- "<hr><br>\n<table>\n \t<tr><th>Sample</th></tr>\n"
        
        pngName <- paste("bic", i, ".png", sep="")
        pngFile <- file.path(outDir, pngName)
        png(pngFile, width=600, height=600)
        bic <- bicluster(resBic, i, graph=TRUE)
        dev.off()
        fileBic <- file.path(outDir, paste("bic", i, ".html",sep=""))
        file.copy(file.path(outDir,"bic.html"), fileBic, overwrite=TRUE)
        contentBic <- readLines(fileBic)
        contentBic <- sub("biclusterName", paste("Bicluster", i, sep=" "), contentBic)
        contentBic <- sub("Graph", pngName, contentBic)
        
        listGenes <- rownames(bic)
        if (modPuce) symbol <- unlist(getSYMBOL(listGenes, annotation(Data)))  
        else symbol <- rep(NA, nrow(bic))
        for (j in 1:length(symbol)){
            tabListGenes <- paste(tabListGenes, "\t<tr> <td>", listGenes[j], "</td> <td>", symbol[j], "</td> </tr>\n", sep="")
        }
        tabListGenes <- paste(tabListGenes, "</table>\n", sep="")                                       
        contentBic <- sub("ListGenes", tabListGenes, contentBic)
        listSamples <- colnames(bic)
        
        for (j in 1:length(listSamples)){
            tabListSamples <- paste(tabListSamples, "\t<tr> <td>",listSamples[j], "</td> </tr>\n",sep="")
        }
        tabListSamples <- paste(tabListSamples, "</table>\n", sep="")                                       
        contentBic <- sub("ListSamples", tabListSamples, contentBic)
        
        write(contentBic, file=fileBic, append=FALSE)
        tab <- paste(tab, "\t<tr>\n\t\t<td><a href=\"bic", i, ".html\">bicluster ", i, "</a></td>\n",sep="")
        for (j in 1:5){
            tab <- paste(tab, "\t\t<td>", round(resBic$mat.resvol.bic[i,j], 3), "</td>\n", sep="")
        }
        tab <- paste(tab, "\t</tr>\n", sep="")
    }
    contentListBic <- sub("TAB", tab, contentListBic)
    write(contentListBic, file=fileListBic, append=FALSE)
    
    geneSet <- resBic$geneSet
    geneSetCol <- geneSet$geneSetCol
    fileGeneSet <- file.path(outDir, "geneSets.html")
    contentGeneSet <- readLines(fileGeneSet)
    if (is.null(geneSet)){
        contentGeneSet <- sub("content", "Analysis not performed", contentGeneSet)
        nbSets <- 0
    } else{
        nbSets <- length(geneSetCol)
        txt <- "\n<table>\n\t<tr>\n\t<th>Gene set</th>\n\t<th>Bicluster</th>\n\t<th>adj p-value</th>\n\t<th>p-value</th></tr>"
        index <- order(geneSet$adjpvalue, na.last=NA)
        
        index <- index[1:100]
        for (i in index){
            n <- floor((i-1)/nbBic)
            m <- round(nbBic * (i/nbBic - n),digits=0)
            nameSet <- setName(geneSetCol[[n+1]])
            
            txt2 <- paste("\n\t<tr>\n\t<td>",nameSet,"</td>\n\t<td><a href=\"bic", m,".html\">Bicluster ", m, "</a></td>\n\t<td>", geneSet$adjpvalue[i], "</td>\n\t<td>", geneSet$pvalue[i],"</td>\n\t</tr>", sep="") 
            txt <- paste(txt, txt2, sep="")
        }
        txt <- paste(txt, "\n</table>\n", sep="")
        contentGeneSet <- sub("content", txt, contentGeneSet)
    }
    contentHome <- sub("NBSETS", nbSets, contentHome)
    contentHome <- sub("ENRICHEDSET", sum(geneSet$adjpvalue<=0.2, na.rm=TRUE), contentHome)
    write(contentGeneSet, fileGeneSet, append=FALSE)
    
    covar <- resBic$covar
    fileCovar <- file.path(outDir, "covar.html")
    contentCovar <- readLines(fileCovar)
    if (is.null(covar)){
        contentCovar <- sub("content", "Analysis not performed", contentCovar)
        nbVar <- 0
    } else {
        nbVar <- length(covar$covar)
        txt <- ""
        txt2 <- "<ul>\n"
        for (i in 1:nbVar) {
            nameCov <-  names(covar$covar)[i]
            nbModa <- length(covar$covar[[i]])
            txt2 <- paste(txt2, "<li><a href=\"#", nameCov, "\">", nameCov,"</a></li>\n", sep="")
            txt <- paste(txt,"<hr><h3><a name=\"", nameCov, "\">", nameCov, "</a></h3>\n", sep="")
            txt <- paste(txt, "\n<table>\n\t<tr>\n\t<th></th>\n\t<th>adjp-value<br><i>(p-value)</i></th>\n", sep="")
            txt3 <- "\t<tr>\n\t<th></th>\n\t<th></th>\n"
            for (j in 1:nbModa){
                txt <- paste(txt, "\t<th>", names(covar$covar[[i]])[j], "</th>\n")
                txt3 <- paste(txt3, "\t<th>", covar$covar[[i]][j], "</th>\n",sep="")
            }
            txt <- paste(txt, "\t</tr>\n")
            txt <- paste(txt, txt3, "\t</tr>\n",sep="")
            pvalOrder <- order(covar$pvalue[,i])
            for (j in pvalOrder){
                txt <- paste(txt, "\t<tr>\n\t<td><a href=\"bic", j,".html\">Bicluster ", j, "</a></td>\n", sep="")
                txt <- paste(txt, "\t<td>", round(covar$adjpvalue[j,i], digits=4), "<br><i>(", round(covar$pvalue[j,i], digits=4), ")</i></td>\n", sep="")
                for (k in 1:nbModa){
                    txt <- paste(txt, "\t<td>", covar$index[[i]][j,k], "<br><i>",round(covar$residuals[[i]][j,k], digits=3),"</i></td>\n", sep="")
                }
                txt <- paste(txt, "\t</tr>\n", sep="")
            }
            txt <- paste(txt, "</table>\n", sep="")
        }
        txt2 <- paste(txt2, "</ul>\n")
        txt <- paste(txt2, txt)
        contentCovar <- sub("content", txt, contentCovar)
    }
    write(contentCovar, file=fileCovar, append=FALSE)
    contentHome <- sub("NBANNOT", nbVar, contentHome)
    contentHome <- sub("ENRICHEDANNOT", sum(covar$adjpvalues<=0.2, na.rm=TRUE), contentHome)

    write(contentHome, file=fileHome, append=FALSE)

    if (browse) browseURL(fileHome, browser = getOption("browser"))
    
}

bicluster <- function (biclustering, k, graph=TRUE)  
{
    if (class(biclustering) != "biclustering")
        stop ("invalid object class, must be of class biclustering")
    idRow <- which(biclustering$bicRow[k,] == 1)
    idCol <- which(biclustering$bicCol[k,] == 1)
    bic <- exprs(biclustering$ExpressionSet)[idRow, idCol]
    rownames(bic) <- featureNames(biclustering$ExpressionSet[idRow, idCol])
    colnames(bic) <- sampleNames(biclustering$ExpressionSet[idRow, idCol])
    class(bic) <- "bicluster"
    if (graph) plot(bic)
    return(bic)
}

testSet <- function(resBic, geneSetCol){
    
    nbBic <- as.integer(resBic$param[1, 2])
    nbSet <- length(geneSetCol)
    
    Data <- resBic$ExpressionSet
    expData <- exprs(Data)
    
    pval <- matrix(data = NA, nrow = nbBic, ncol = nbSet)
    inter <- matrix(data = NA, nrow = nbBic, ncol = nbSet)  
    setSize <- vector(length=nbSet)
    nbTest <- 0
    
    setPop <- GeneSet(Data, setName="pop")
    idTypePop <- geneIdType(setPop)
    dimPop <- length(geneIds(setPop))
    
    for (i in 1:nbSet){
        set <- geneSetCol[[i]]
        idTypeSet <- geneIdType(set)
        
        if (attributes(idTypeSet)$type != attributes(idTypePop)$type) setMapped <- mapIdentifiers(set, idTypePop)
        else setMapped <- set
        adjSet <- setPop & setMapped
        dimSet <- length(geneIds(adjSet))
        dimNonSet <- dimPop - dimSet
        
        for (j in 1:nbBic){
            bic <- bicluster(resBic, j, graph=FALSE)
            bic <- Data[rownames(bic),]
            setBic <- GeneSet(bic, setName="bic")
            dimBic <- length(geneIds(setBic))
            adjBic <- adjSet & setBic
            dimAdjBic <- length(geneIds(adjBic))
            
            if (dimAdjBic > 0){
                pval[j, i] <- phyper(dimAdjBic, dimSet, dimNonSet, dimBic, lower.tail=FALSE)
            }
        }
    }
    
    adjpval <- mt.rawp2adjp(pval , proc="BY")
    adjpval <- matrix(data=adjpval$adjp[order(adjpval$index),2], nrow=nrow(pval), ncol=ncol(pval))
        
    res <- list(geneSetCol,pval,adjpval)
    names(res) <- c("geneSetCollection", "pvalues", "adjpvalues")
    class(res) <- "testSet"
    resBic$geneSet <- res
    return(resBic)
}

print.biclustering <-  function(x, ...){
    param <- x$param
    resVol <- x$mat.resvol.bic
    cl <- x$Call
    cat("\nCall : \n")
    print(cl)
    cat("\nParameters :")
    cat("\n number of biclusters : ", param[1,2])
    cat("\n residu threshold : ", param[2,2])
    cat("\n gene initial probability : ", param[3,2])
    cat("\n sample initial probability : ", param[4,2])
    cat("\n number of iterations : ", param[5,2])
    cat("\n date : ", param[6,2])    
    cat("\nBiclusters : \n")
    print(resVol, quote=FALSE)
    if (!is.null(x$covar) || !is.null(x$geneSet)) cat("\nAdditionnal analyses :\n")
    if (!is.null(x$covar)) cat("\t Sample covariates over-representation\n")
    if(!is.null(x$geneSet)) cat("\t Gene sets over-representation\n")
}

plot.bicluster <- function(x, ...){
    op <- par(mar = c(10,4,4,2))
    matplot(t(x), type = "b", pch = 1, ylab = "expression level", axes = FALSE, ...)
    axis(1, at = 1:ncol(x), labels = colnames(x), las = 2, cex.axis = 0.75)
    axis(2)
    par(op)
}

testAnnot <- function(resBic, annot=NULL, covariates="all"){
    
    if (class(resBic)!="biclustering") stop("Not a biclustering object")
    nbBic <- as.integer(resBic$param[1, 2])
    ow <- options("warn")
    bicCol <- resBic$bicCol
    
    Data <- resBic$ExpressionSet
    if (is.null(annot)){
        annot <- pData(phenoData(Data))
        if (is.null(annot)) stop("No covariates provided !")
    }
    annot <- as.data.frame(annot)
    
    if ((length(covariates) == 1) && (covariates == "all")){
        whichVar <- 1:ncol(annot)
    } else whichVar <- match(covariates, colnames(annot))
    whichVar <- whichVar[!is.na(whichVar)]
    nbVar <- length(whichVar)
    
    pval <- matrix(nrow=nbBic, ncol=nbVar)
    colnames(pval) <- colnames(annot)[whichVar]
    index <- vector(length=nbVar, mode="list")
    resid <- vector(length=nbVar, mode="list")
    adjpvalue <- pval
    
    listVar <- vector(length=nbVar, mode="list")
    
    for (i in 1:nbVar){
        covar <- annot[, whichVar[i]]
        listVar[[i]] <- as.vector(summary.factor(covar))
        names(listVar[[i]]) <- levels(covar)
        names(listVar)[i] <- colnames(annot)[i]
        resid[[i]] <- matrix(nrow=nbBic, ncol=length(listVar[[i]]))
        index[[i]] <- matrix(nrow=nbBic, ncol=length(listVar[[i]]))
        names(index)[i] <- names(resid)[i] <- colnames(annot)[i]
        p <- listVar[[i]]
        idCol <- whichVar[i]
        colnames(resid[[i]]) <- colnames(index[[i]]) <- levels(covar)
        
        
        for (j in 1:nbBic){
            samples <- which(bicCol[j,]==1)
            x <- as.vector(summary(annot[samples, idCol]))
            
            options(warn=-1)
            test <- chisq.test(x, p=p, rescale.p=TRUE, simulate.p.value=FALSE)
            options(ow)
            
            pval[j,i] <- test$p.value
            resid[[i]][j,] <- test$residuals
            index[[i]][j,] <- x
            
        }
        adjpval <- mt.rawp2adjp(pval[,i] , proc="BY")
        adjpvalue[,i] <- matrix(data=adjpval$adjp[order(adjpval$index),2], nrow=nrow(pval), ncol=1)
    }
    colnames(adjpvalue) <- colnames(annot)[whichVar]
    
    res <- list(listVar, pval, adjpvalue, index, resid)
    names(res) <- c("covar", "pvalues", "adjpvalues", "index", "residuals")
    resBic$covar <- res
    return(resBic)
}

residue <- function(Data)
{
    if (class(Data)!= "ExpressionSet") Data <- new("ExpressionSet", exprs=Data)
    expData <- exprs(Data)
    vecData <- as.vector(t(expData))
    bic.row <- rep(1, nrow(expData))
    bic.col <- rep(1, ncol(expData))
    resid <- .C("printres", as.integer(nrow(expData)), as.integer(ncol(expData)), as.double(vecData), as.integer(bic.row), as.integer(bic.col), res=double(1),PACKAGE="BicARE")$res
    return(resid)
}

FLOC <- function(Data, k=20, pGene=0.5, pSample=pGene, r=NULL, N=8, M=6, t=500, blocGene=NULL, blocSample=NULL)
{
    if (class(Data) != "ExpressionSet") Data <- new("ExpressionSet", exprs=Data)
    expData <- exprs(Data)
    
    if (is.null(r)) r <- residue(expData)/10
    
    cl <- match.call()
    
    param <- matrix(nrow=6, ncol=2)
    param[,1] <- c("number of bicluster", "residu threshold", "genes initial probability", "samples initial probability","number of iterations", "date")
    param[,2] <- c(k, r, pGene, pSample, t, date())
    
    vecData <- as.vector(t(expData))
    nbGenes <- nrow(expData)
    nbSamples <- ncol(expData)
    
    if (!is.null(blocGene) || !is.null(blocSample)) k <- max(ncol(blocGene), ncol(blocSample), k)
    vecBicRow <- matrix(data=0, nrow=nbGenes, ncol=k)
    vecBicCol <- matrix(data=0, nrow=nbSamples, ncol=k)
    
    if (is.null(blocGene)) blocGene <- matrix(data=0, nrow=nbGenes, ncol=k)
    else vecBicRow[,1:ncol(blocGene)] <- blocGene
    vecBlocGene <- vecBicRow <- as.vector(vecBicRow)
    
    if (is.null(blocSample)) blocSample <- matrix(data=0, nrow=nbSamples, ncol=k)
    else vecBicCol[,1:ncol(blocSample)] <- blocSample
    vecBlocSample <- vecBicCol <- as.vector(vecBicCol)                 
    
    rand1 <- runif(k * nbGenes)
    rand2 <- runif(k * nbSamples)
    
    vecBicRow[which(rand1 < pGene)] <- 1
    vecBicCol[which(rand2 < pSample)] <- 1
    
    vec.resvol.bic <- vector(mode="double", length = k * 4)
    
    res.floc <- .C("floc", vecData = as.double(vecData), as.integer(nbGenes), as.integer(nbSamples), vecBicRow = as.integer(vecBicRow), vecBicCol = as.integer(vecBicCol), vec.resvol.bic = as.double(vec.resvol.bic), as.double(r), as.integer(k),as.integer(N), as.integer(M), as.integer(t), as.integer(vecBlocGene), as.integer(vecBlocSample), PACKAGE="BicARE")
    
    bicRow <- matrix(data = res.floc$vecBicRow, nrow = k, ncol = nbGenes, byrow = TRUE)
    bicCol <- matrix(data = res.floc$vecBicCol, nrow = k, ncol = nbSamples, byrow = TRUE)
    
    mat.resvol.bic <- matrix(data = 0, nrow = k, ncol = 5)
    mat.resvol.bic[,1:4] <- matrix(data=res.floc$vec.resvol.bic, nrow = k, ncol = 4, byrow = TRUE)[,1:4]
    for (i in 1:k) mat.resvol.bic[i,5] <- mean(apply(expData[which(bicRow[i,] == 1), which(bicCol[i,] == 1)], 1, var))
    
    colnames(mat.resvol.bic) <- c("residue", "volume", "genes", "conditions", "rowvar")
    
    resBic <- list(cl, Data, param, bicRow, bicCol, mat.resvol.bic)
    names(resBic) <- c("Call", "ExpressionSet", "param", "bicRow", "bicCol", "mat.resvol.bic")
    
    class(resBic) <- "biclustering"
    return(resBic)
}

% \VignetteIndexEntry{BicARE}
% \VignetteDepends{}
% \VignetteKeywords{Biclustering Analysis}
% \VignettePackage{BicARE}

\documentclass[11pt]{article}

\usepackage{amsmath}
\usepackage[authoryear,round]{natbib}
\usepackage{hyperref}
\usepackage{geometry}
\geometry{verbose,letterpaper,tmargin=20mm,bmargin=20mm,lmargin=2.5cm,rmargin=2.5cm}
\SweaveOpts{echo=FALSE}
% -----------------------------------------------------------------------------------------------
\begin{document}

\title{\bf BicARE : Biclustering Analysis and Results Exploration}

\author{Pierre Gestraud$^{1,2,3}$, Isabel Brito$^{1,2,3}$ and Emmanuel Barillot$^{1,2,3}$}

\maketitle

\begin{center}
  1. Institut Curie, Paris, F-75248 France
  
  2. INSERM, U900, Paris, F-75248 France
  
  3. Ecole des Mines de Paris, Fontainebleau, F-77300 France
  
  \textit{http://bioinfo.curie.fr}
\end{center}

\tableofcontents

\section{Overview}

This document presents an overview of the \textit{BicARE} package. This package is dedicated to biclustering analysis which allows to discover sets of genes that have the same expression pattern accross a set of samples (for an overview of bicluster analysis see \cite{Madeira2004}). 

\section{Biclustering analysis}

<<echo=FALSE, print=FALSE>>=
require(BicARE)
@ 

Data set used in this vignette is a subset of the \emph{sample.ExpressionSet}, 26 hgu95av2 arrays normalised by dChip. Only 352 probesets are kept (probesets with a minimal value greater than 1) and expression values are set to log2 scale. 

<<echo=TRUE, print=TRUE>>=
data(sample.bicData)
sample.bicData
@ 

The biclustering algorithm used in \emph{BicARE} is based on the notion of residue which is a measure of coherence of the elements in a bicluster (see \cite{Yang2005} for a definition of the residue). The smaller the residue, the more coherent the bicluster.

Computing the residue of the data matrix :
<<echo=TRUE, print=TRUE>>=
residue(sample.bicData)  
@ 
The core of the package is the \emph{FLOC} function which launches a modified version of the FLOC (FLexible Overlapped biClustering) algorithm, see \cite{Yang2005}. A predetermined number (parameter \emph{k}) of biclusters are build such that they are as big as possible with a residue smaller than the threshold (parameter \emph{r}). 

\subsection{Random intialisation}

In a strictly exploratory approach it is possible to build the biclusters around random seeds (random biclusters that are iteratively improved). The size of the random seeds is controlled by parameters \emph{pGene} and \emph{pSample} such as each gene (sample) has a probability \emph{pGene} (\emph{pSample}) to belong to each bicluster. Other parameters are the number of biclusters build (\emph{k}), the residue threshold (\emph{r}), the minimal number of genes (\emph{N}) per bicluster, the minimal number of conditions (\emph{M}) per bicluster and the number of iterations (\emph{t}).

<<echo=TRUE, print=FALSE>>=
set.seed(1)
res.biclustering <- FLOC(sample.bicData, k=15, pGene=0.3, pSample=0.6, r=0.01, 10, 8, 200)
@
<<echo=TRUE, print=TRUE>>=
res.biclustering
@

A data frame (\emph{mat.resvol.bic}) gives the characteristics of the biclusters such as the residue or the size of the biclusters. In the \emph{rowvar} column is displayed the mean of the variance of the genes in the bicluster. It helps to find non-trivial biclusters (e.g. constant bicluster). 

\subsection{Directed initialisation}
It is also possible to build biclusters around previous knowledge. Initialisation can be performed on genes, on samples or on both.

The initialisation is performed by building boolean matrices indicating the membership of the elements to the biclusters. 

<<echo=TRUE, print=FALSE>>=
init.genes <- matrix(data=0, nrow=352, ncol=5)
init.samples <- matrix(data=0, nrow=26, ncol=5)
init.genes[1:10,1] <- 1
init.genes[20:30,2] <- 1
init.genes[50:60,3] <- 1
init.samples[1:5,3] <- 1
init.samples[1:5,4] <- 1
init.samples[10:15,5] <- 1
@ 

Here the five first biclusters are initialised. Biclusters 1 and 2 are respectively initialised around genes 1 to 10 and 20 to 30; biclusters 4 and 5 around samples 1 to 5 and 10 to 15; bicluster 3 is initialised around genes 50 to 60 and samples 1 to 5. 

If the number of biclusters initialised is different from the parameter \emph{k}, the real number of biclusters searched is the greater of the two. 

The two initialisation matrices are given as arguments \emph{blocGene} and \emph{blocSample} to the \emph{FLOC} function.

\subsection{Bicluster extraction}

The \emph{bicluster} function extracts a bicluster from a \emph{biclustering} object. The bicluster is returned as a matrix with the genes on rows and the conditions on columns. The \emph{graph} option determines if a graphic should be plotted. 

<<echo=TRUE, print=FALSE, fig=TRUE>>=
bic <- bicluster(res.biclustering, 6, graph=FALSE)
plot(bic)
@ 
\section{Additionnal analyses}

Additionnal analyses can be performed on the biclustering results. These analyses allows a functionnal view of the biclusters by testing the over-representation of gene sets or a characterisation of the samples (e.g. clinical covariates).

\subsection{Genesets enrichment}
A functionnal view of the biclusters can be obtained by testing the over-representation of a priori defined gene sets. This over-representation is evaluated by an hypergometric test. The gene sets are in the \emph{GeneSetCollection} format.

<<echo=TRUE, print=FALSE>>=
gsc <- GeneSetCollection(res.biclustering$ExpressionSet[1:50], setType=GOCollection())
res.bic2 <- testSet(res.biclustering, gsc)
@

The \emph{testSet} function returns an updated \emph{biclustering} object with a new attribute \emph{geneSet}. It is a list containing the \emph{GeneSetCollection} used, the p-values (from an hypergeometric test) and the adjusted p-values.

\subsection{Sample covariates enrichment}

Three covariates for the samples are provided in the example data set. Function \emph{testAnnot} only uses categorical covariates (here \emph{sex} and \emph{type}). For each bicluster, the function evaluates, with a $\chi^2$ test of adequation, the enrichment of each level of the covariates. 

<<echo=TRUE, print=FALSE>>=
pData(sample.bicData)
@ 

<<echo=TRUE, print=FALSE>>=
res.bic2 <- testAnnot(res.biclustering, annot=pData(sample.bicData), covariates=c("sex", "type"))
@

As \emph{testSet} does, the \emph{testAnnot} function returns an updated \emph{biclustering} object with a new attribute \emph{covar}. It is a list containing the covariates used, the p-values and adjusted p-values, the numbers of each level in each bicluster and the residuals of the tests.

\section{Building html report}

Dealing with biclustering results is a tedious task because of the amount of data to explore. A user-friendly report can be created with the function \emph{makeReport}. It allows an easy navigation through the results by different approaches : 
\begin{itemize}
\item by bicluster
\item by gene sets in order to examine the biclusters enriched in gene sets of interest
\item by sample covariates in order to examine the biclusters enriched in samples with characteristics of interest
\end{itemize}

Each bicluster is presented by the lists of the genes and samples and by a plot of the expression profiles.

\begin{Schunk}
  \begin{Sinput}
    > dirPath <- getwd()
    > dirName <- "test"
    > makeReport(dirPath=dirPath, dirName=dirName, resBic=res.bic2, browse=FALSE)
  \end{Sinput}
\end{Schunk}

The html report is created in the directory \emph{dirName} in the current working directory. The \emph{browse} option determines if the web browser is opened on the \textit{home.html} file. Before running \emph{makeReport} it is advised to check if the results directory can be created.   

\bibliographystyle{apalike}
\bibliography{biblio}

\end{document}

\name{sample.biclustering}
\docType{data}
\alias{sample.biclustering}

\title{Example biclustering object}

\description{A biclustering object created by the \code{\link{FLOC}}
  function on the sample.bicData with the following options
  : k=10, pGene = 0.3, pSample = 0.5, 
    r = 0.025, N = 8, M = 8, t = 1000.
}

\usage{sample.biclustering}

\format{a biclustering object}

\keyword{datasets}

\name{makeReport}
\alias{makeReport}

\title{Export the results as html files}

\description{Creates a directory with html files containing the
  biclustering results.}

\usage{
makeReport(dirPath, dirName, resBic, browse=TRUE)
}


\arguments{
  \item{dirPath}{path to the directory}
  \item{dirName}{the name of the directory where the report will be created}
  \item{resBic}{a biclustering result}
  \item{browse}{logical. If TRUE the web browser will be opened}
}

\details{
  \code{makeReport} produces a html report of biclustering results in a new directory named \code{dirName}.  
  If the \code{browse} argument is set to TRUE the web browser will be opened on the
  "home.html" file. 
  
  Make sure to have rights to create the result directory.
}

\author{
  Pierre Gestraud \email{pierre.gestraud@curie.fr}
}

\examples{

data(sample.biclustering)
dirPath <- getwd() ## report created in the current working directory
dirName <- "test"
makeReport(dirPath, dirName, sample.biclustering, browse=FALSE)
}

\keyword{cluster}

\name{BicARE-package}
\alias{BicARE-package}
\alias{BicARE}
\docType{package}
\title{
BicARE
}
\description{
Biclustering Analysis and Results Exploration
}
\details{
\tabular{ll}{
Package: \tab BicARE\cr
Version: \tab 0.1.0\cr
Date: \tab 2008-06-05\cr
Depends: \tab R (>= 1.8.0), Biobase, multtest, GSEABase\cr
License: \tab GPL\cr
biocViews: \tab Microarray, Transcription, Statistics, Clustering\cr
URL: \tab http://bioinfo.curie.fr\cr
Packaged: \tab Tue Aug 19 14:21:38 2008; pgestraud\cr
Built: \tab R 2.8.0; i686-pc-linux-gnu; 2008-08-19 14:39:57; unix\cr
}

Index:
\preformatted{
FLOC                    Performs the FLOC algorithm
bicluster               Extract a bicluster
makeReport              Export the results as html files
residue                 Residue of a matrix
sample.bicData          Example data set for BicARE
sample.biclustering     Example biclustering object
testAnnot               Find samples annotations over-represented in
                        biclusters
testSet                 Find gene sets that are enriched in a bicluster
}

Further information is available in the following vignettes:
\tabular{ll}{
\code{BicARE} \tab BicARE (source, pdf)\cr
}
}
\author{
Pierre Gestraud

Maintainer: Pierre Gestraud , <pierre.gestraud@curie.fr>
}
\keyword{ package }

\name{testSet}
\alias{testSet}
\title{Find gene sets that are enriched in a bicluster}
\description{Test of the over-representation of gene sets in the biclusters}
\usage{testSet(resBic, geneSetCol)}

\arguments{
  \item{resBic}{a \code{biclustering} object created by \code{\link{FLOC}}}
  \item{geneSetCol}{a \code{\link[GSEABase]{GeneSetCollection-class}}}
}

\details{
	The over-representation of a gene set in a bicluster is evaluated by an hypergeometric test. 
	    
	The genes identifiers of the gene sets will automatically be mapped to
  	the same as those used in the data. 
  
  	Due to the amount of results it is advised to use the \code{\link{makeReport}} function to get a html report. 
}

\value{
  A \code{biclustering} object containing \code{resBic} and updated with
  the results of the tests in \code{resBic$geneSet}.
  
  The results are presented as a list with : 
  \item{GeneSetCollection}{the GeneSetCollection used}
  \item{pvalues}{a matrix containing the pvalues of the tests for each geneSet and each bicluster}
  \item{adjpvalue}{a matrix containing the p-values adjusted by the Benjamini Yekutieli procedure}
}

\author{Pierre Gestraud \email{pierre.gestraud@curie.fr}}

\examples{
data(sample.biclustering)
gss <- GeneSetCollection(sample.biclustering$ExpressionSet[1:50,], setType=GOCollection())
resBic <- testSet(sample.biclustering, gss)
}

\keyword{cluster}

\name{testAnnot}

\alias{testAnnot}

\title{Find samples annotations over-represented covariates in biclusters}

\description{Characterisation of the biclusters in term of over-representation of sample covariates.}

\usage{testAnnot(resBic, annot=NULL, covariates="all")}

\arguments{
	\item{resBic}{a biclustering result from \link{FLOC}}
  	\item{annot}{annotation matrix, default value is set to NULL, then phenoData of the ExpressionSet is used}
  	\item{covariates}{the names of the covariates that should be tested,
    default value is set to "all"}
}

\details{
  	For each bicluster and each covariate a chi-squarred test is performed to test the adequation between the distribution of the levels of the covariates in the bicluster and in the original dataset.  
  
	Multiple testing correction is performed by the Benjamini-Yekutieli procedure. The residuals of the tests indicate if the level is over or down represented in the bicluster.  
    
    Due to the amount of results it is advised to use the \code{\link{makeReport}} function to get a html report. 
}

\value{
	A \code{biclustering} object containing \code{resBic} and updated with
  	the results of the tests in \code{resBic$covar}.
 
  	The results are presented as a list with : 
  	\item{covar}{the samples covariates tested}
  	\item{pvalues}{a matrix with the p-values of the tests}
  	\item{adjpvalues}{a matrix with the p-values adjusted by the Benjamini Yekutieli procedure}
  	\item{index}{a list of matrices with the numbers of each level in each bicluster}
  	\item{residuals}{a list of matrices with the residuals of the tests for each modality in each bicluster}
}
\author{Pierre Gestraud}

 	
\examples{
data(sample.biclustering)
resBic <- testAnnot(sample.biclustering, annot=NULL, covariates=c("sex", "type"))
}

\keyword{cluster}

\name{sample.bicData}
\docType{data}
\alias{sample.bicData}

\title{Example data set for BicARE}

\description{A subset of sample.ExpressionSet from package Biobase. The data for 26 cases, labeled A to Z and 350 genes. Each case has
  three covariates: sex (male/female), type (case/control) and
  score (testing score).
}

\usage{sample.bicData}

\format{An ExpressionSet}

\keyword{datasets}

\name{bicluster}
\alias{bicluster}
\title{Extract a bicluster}

\description{Extract a bicluster from an object of class biclustering}

\usage{bicluster(biclustering, k, graph=TRUE)}

\arguments{
  \item{biclustering}{an object of class "biclustering" created by
    function \code{\link{FLOC}}}
  \item{k}{the number of the bicluster considered in the "biclustering" object}
  \item{graph}{boolean, indicating whether the graph should be plotted
    or not}
}

\value{Returns the bicluster as a matrix with the genes on rows and the
  samples on columns. Result matrix is of class "bicluster". The "graph" option allows to plot the expression profiles of the genes across the conditions in the bicluster.}

\author{Pierre Gestraud}

\examples{
### extract the first bicluster
data(sample.biclustering)
sample.biclustering
bic <- bicluster(sample.biclustering, 1, graph=TRUE)
plot(bic)
}

\keyword{cluster}

\name{FLOC}

\alias{FLOC}

\title{Performs the FLOC algorithm}

\description{Find a given number of biclusters using the a modified
  version of the FLOC algorithm.} 

\usage{FLOC(Data, k = 20, pGene = 0.5, pSample=pGene, r = NULL, N = 8, M
= 6, t = 500, blocGene = NULL, blocSample = NULL)} 

\arguments{
   \item{Data}{an
   \code{\link[Biobase:class.ExpressionSet]{ExpressionSet}} or a
   matrix (with genes on rows and conditions on columns)}
   \item{k}{the number of biclusters searched}
   \item{pGene}{genes initial probability of membership to the biclusters}
   \item{pSample}{samples initial probability of membership to the biclusters}
   \item{r}{the residue threshold}
   \item{N}{minimal number of gene per bicluster}
   \item{M}{minimal number of conditions per bicluster}
   \item{t}{number of iterations}
   \item{blocGene}{a matrix indicating the directed initialisation for
   the genes (see details)}
   \item{blocSample}{a matrix indicating the directed initialisation for
   the conditions (see details)}
}

\details{This biclustering algorithm is based on the FLOC algorithm (FLexible
  Overlapped biClustering) defined by Yang et al. (see references). It
  can discover a set of \code{k}, possibly overlapping, biclusters. If
  \code{r} is set to 
  NULL, the residue threshold used in the analysis is the residue of
  \code{Data} divided by 10.
  
  \code{blocGene} and \code{blocSample} are matrix of 0 and 1 with the rows
  representing the features (gene or samples) and the columns the
  biclusters. A 1 on line i and column j indicates that the feature i
  (gene or sample) will be include in the bicluster j during the
  initialisation step and will not be removed from it during the analysis. If the number of columns in these matrices is different from the number of bicluster searched, \code{k} is set to the maximal value
  of these two.
  
  See \code{\link{bicluster}} to extract a bicluster from the biclustering result. 
}

\value{
  Returns an object of class 'biclustering', a list containing at
  least :
  \item{Call}{the matched call.}
  \item{ExpressionSet}{the data used}
  \item{param}{a data.frame with the algorithm parameters}
  \item{bicRow}{a matrix of boolean indicating the belonging of the
    genes to the biclusters}
  \item{bicCol}{the same as for bicRow but for the conditions}
  \item{mat.resvol.bic}{a matrix describing the biclusters}
}

\references{J. Yang, H. Wang, W. Wang, and P.S. Yu. An improved biclustering method for analyzing gene expression. International Journal on Artificial Intelligence Tools, 14(5):771-789, 2005}

\author{
  Pierre Gestraud (\email{pierre.gestraud@curie.fr})
}

\examples{

data(sample.bicData)     ## subset of sample.ExpressionSet from Biobase
residue(sample.bicData)  ##  0.3401921
resBic <- FLOC(sample.bicData, k=10, pGene=0.5,r=0.05,N=8,M=10,t=500)
resBic

## initialising samples of 2 biclusters
iniSample <- matrix(0, ncol=2, nrow=26)
## first bicluster initialised around Female cases
iniSample[pData(sample.bicData)$sex=="Female",1] <- 1
## second bicluster initialised around control cases
iniSample[pData(sample.bicData)$type=="Control",2] <- 1
resBic <- FLOC(sample.bicData, k=10, pGene=0.5, r=0.05, N=8, M=10, t=500, blocSample=iniSample)
resBic
}


\keyword{cluster}

\name{residue}

\alias{residue}

\title{Residue of a matrix}

\description{Returns the residue of a matrix.}

\usage{residue(Data)}

\arguments{
   \item{Data}{an
   \code{\link[Biobase:class.ExpressionSet]{ExpressionSet-class}} or a
   matrix} 
}

\details{
  This function computes the residue of a matrix as defined
  by Yang et al (see references). 
}

\references{J. Yang, H. Wang, W. Wang, and P.S. Yu. An improved
  biclustering method for analyzing gene expression. International
  Journal on Artificial Intelligence Tools, 14(5):771-789, 2005} 

\author{
  Pierre Gestraud
}

\seealso{
  \code{\link{FLOC}}
}

\examples{
data(sample.bicData)
residue(sample.bicData)
}

\keyword{cluster}
